<?php

namespace Drupal\vev_connector\Controller;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\RevisionLogInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\media\Entity\Media;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class VevWebhook.
 */
class VevWebhook extends ControllerBase {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The mail manager.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The configuration object factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Media entity.
   *
   * @var \Drupal\media\Entity\Media
   */
  protected $media;

  /**
   * Class constructor.
   */
  public function __construct(
    AccountProxyInterface $current_user,
    TimeInterface $time,
    MailManagerInterface $mail_manager,
    ConfigFactoryInterface $config_factory
  ) {
    $this->currentUser = $current_user;
    $this->time = $time;
    $this->mailManager = $mail_manager;
    $this->configFactory = $config_factory;

    $this->media = $this->entityTypeManager()->getStorage('media');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('datetime.time'),
      $container->get('plugin.manager.mail'),
      $container->get('config.factory'),
    );
  }

  /**
   * Vev webhook receiver.
   */
  public function receiver(Request $request) {
    $vev_connector_settings = $this->configFactory->get('vev_connector.settings');
    $key = $vev_connector_settings->get('vev_key');

    if (!empty($key)) {
      $data = Json::decode($request->getContent());

      $signature = $request->headers->get('x-vev-signature');
      $signature_data = explode('=', $signature);
      $algo = $signature_data[0] ?: 'sha512';
      $hash = hash_hmac($algo, $request->getContent(), mb_convert_encoding($key, 'UTF-8'));

      if (!empty($data) && $data['event'] === 'PUBLISH' && $signature_data[1] === $hash) {
        $media = [];
        $updated_media = [];
        foreach ($data['payload']['pages'] as $page) {
          $identificator = sprintf('%s-%s', $data['payload']['projectId'], $page['created']);
          $headers = get_headers($page['downloadUrl']);
          $vev = [
            'value' => strpos($headers[0],'200') ? file_get_contents($page['downloadUrl']) : '',
            'format' => 'full_html'
          ];
          $media = $this->media->loadByProperties(['field_identificator' => $identificator]);
          if (!empty($media) && is_array($media)) {
            $media = array_pop($media);
            if ($media instanceof Media) {
              $name = sprintf('%s - %s - %s', $data['payload']['projectTitle'], $page['title'], $data['payload']['projectId']);

              $media->set('name', $name);
              $media->set('field_vev_component', $vev);

              if (!empty($page['cover'])) {
                $media->set('field_thumbnail', $page['cover']);
              }

              if ($media->hasField('moderation_state')) {
                $media->set('moderation_state', 'draft');
              }
              $media->save();

              $url = $request->getSchemeAndHttpHost() . '/media/' . $media->id() . '/edit';
              $updated_media[] = $url;

              if ($media->hasField('moderation_state')) {
                $entity = $this->media->createRevision($media, $media->isDefaultRevision());
                if ($entity instanceof RevisionLogInterface) {
                  $entity->set('moderation_state', 'draft');
                  $entity->setRevisionCreationTime($this->time->getRequestTime());
                  $entity->setRevisionLogMessage('Draft created automatically');
                  $entity->setRevisionUserId($this->currentUser->id());
                  $entity->save();
                }
              }

              $this
                ->getLogger('vev_connector')
                ->info(
                  'Vev media: "%data" has been updated',
                  ['%data' => $name]
                );

              $this
                ->getLogger('vev_connector')
                ->info(
                  'Vev url: %data',
                  ['%data' => $url]
                );
            }
          }
          else {
            $name = sprintf('%s - %s - %s', $data['payload']['projectTitle'], $page['title'], $data['payload']['projectId']);

            $media = $this->media->create([
              'bundle' => 'vev_api',
              'name' => $name,
              'field_identificator' => $identificator,
              'field_vev_component' => $vev,
              'field_thumbnail' => empty($page['cover']) ? '' : $page['cover'],
              'moderation_state' => 'published',
            ]);
            $media->save();

            $this
              ->getLogger('vev_connector')
              ->info(
                'Vev media: "%data" has been created',
                ['%data' => $name]
              );
          }
        }

        $email = $vev_connector_settings->get('vev_notification_email');
        if (
          $vev_connector_settings->get('vev_notification') === 1 &&
          $updated_media && $email && $media instanceof Media
        ) {
          $list = implode(', ', $updated_media);

          $system_site_config = $this->configFactory->get('system.site');
          $module = 'vev_connector';
          $key = 'vev_connector';
          $params['to'] = $email;
          $params['medias'] = $list;
          $params['senders_email'] = $system_site_config->get('mail');
          $result = $this->mailManager->mail(
            $module,
            $key,
            $email,
            $this->currentUser->getPreferredLangcode(),
            $params
          );

          if ($result['result'] != TRUE) {
            $this
              ->getLogger('vev_connector')
              ->error('Failed to send Vev notification mail.');
          }
          else {
            $this
              ->getLogger('vev_connector')
              ->info(
                'Vev notification email was sent (Updated media: %data).',
                ['%data' => $list]
              );
          }
        }

        return new Response('We received your webhook', 200);
      }
      return new Response('Media was not updated', 406);
    }
    else {
      return new Response('Vev secret key is not configured yet!', 406);
    }
  }

}
