<?php

namespace Drupal\vev_connector\Form;

use Drupal;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Setting form for Email Template.
 */
class VevConnectorSettings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'vev_connector_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['vev_connector.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('vev_connector.settings');

    $form['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Vev secret token'),
      '#required' => TRUE,
      '#default_value' => $config->get('vev_key') ?: '',
      '#description' => $this->t('You can generate the secret token for the Vev Webhook here:<br><a href="https://help.vev.design/en/articles/6165061-webhook-integrations#h_bdc9068eec" target="_blank">https://help.vev.design/en/articles/6165061-webhook-integrations#h_bdc9068eec</a>'),
    ];

    $form['group'] = [
      '#type' => 'fieldset',
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#title' => $this->t('Vev media email notifications'),
    ];

    $form['text'] = [
      '#type' => 'item',
      '#group' => 'group',
      '#markup' => $this->t('The Vev Connector module can optionally send email notifications whenever Vev content is pushed into the Media Library.'),
    ];

    $form['checkbox'] = [
      '#type' => 'checkbox',
      '#group' => 'group',
      '#title' => $this->t('Send email notification when Vev media arrives'),
      '#default_value' => $config->get('vev_notification'),
    ];

    $form['email'] = [
      '#type' => 'textarea',
      '#group' => 'group',
      '#title' => $this->t('Notification email recipients'),
      '#default_value' => $config->get('vev_notification_email') ?: '',
      '#description' => $this->t('Separate multiple emails with a comma.'),
      '#states' => [
        'enabled' => [
          ':input[name="checkbox"]' => [
            'checked' => TRUE
          ],
        ],
        'required' => [
          ':input[name="checkbox"]' => [
            'checked' => TRUE
          ],
        ],
      ],
    ];

    $form['subject'] = [
      '#type' => 'textfield',
      '#group' => 'group',
      '#title' => $this->t('Notification subject'),
      '#default_value' => $config->get('vev_notification_email_subject') ?: $this->t('Vev media has arrived on [site:name]'),
      '#description' => $this->t('Email subject for Vev media notifications.'),
      '#states' => [
        'enabled' => [
          ':input[name="checkbox"]' => [
            'checked' => TRUE
          ],
        ],
        'required' => [
          ':input[name="checkbox"]' => [
            'checked' => TRUE
          ],
        ],
      ],
    ];

    $form['body'] = [
      '#type' => 'textarea',
      '#group' => 'group',
      '#title' => $this->t('Notification email body'),
      '#default_value' => $config->get('vev_notification_email_body') ?: $this->t('New or updated Vev media has arrived:<br>[link-to-media-entity]'),
      '#description' => $this->t('Email body for Vev media notifications, Tokens can be used.'),
      '#states' => [
        'enabled' => [
          ':input[name="checkbox"]' => [
            'checked' => TRUE
          ],
        ],
        'required' => [
          ':input[name="checkbox"]' => [
            'checked' => TRUE
          ],
        ],
      ]
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state)
  {
    parent::validateForm($form, $form_state);

    $email_string = $form_state->getValue('email');
    if (!empty($email_string)) {
      $emails = explode(',', $email_string);

      foreach ($emails as $email) {
        if (!filter_var(trim($email), FILTER_VALIDATE_EMAIL)) {
          $form_state->setErrorByName('email', $this->t('Enter a valid email addresses, separating each emails with a comma.'));
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $token = Drupal::token();

    $this->config('vev_connector.settings')
      ->set('vev_key', $form_state->getValue('key'))
      ->set('vev_notification', $form_state->getValue('checkbox'))
      ->set('vev_notification_email', $form_state->getValue('email'))
      ->set('vev_notification_email_subject', $token->replace($form_state->getValue('subject')))
      ->set('vev_notification_email_body', $form_state->getValue('body'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
