<?php

namespace Drupal\vev_connector\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Utility\Token;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'html' formatter.
 */
#[FieldFormatter(
  id: 'html',
  label: new TranslatableMarkup('HTML'),
  field_types: [
    'text',
    'text_long',
    'text_with_summary',
    'string',
    'string_long',
  ],
)]
class HtmlFormatter extends FormatterBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The token service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * The route match service.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Constructs a StringFormatter instance.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Utility\Token $token
   *    The token service.
   * @param \Drupal\Core\Routing\RouteMatchInterface $current_route_match
   *    The route match service.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *    The current user.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    EntityTypeManagerInterface $entity_type_manager,
    Token $token,
    RouteMatchInterface $current_route_match,
    AccountProxyInterface $current_user
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);

    $this->entityTypeManager = $entity_type_manager;
    $this->token = $token;
    $this->routeMatch = $current_route_match;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager'),
      $container->get('token'),
      $container->get('current_route_match'),
      $container->get('current_user'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    return [
        'allowed_tags' => [],
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    $form['allowed_tags'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Allowed tags'),
      '#description' => $this->t('Specify allowed tags. Enter one tag per line.'),
      '#default_value' => !empty($this->getAllowedTags()) ? implode("\r\n", $this->getAllowedTags()) : '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    if (!empty($this->getAllowedTags())) {
      $summary[] = $this->t('Allowed tags: %tags', ['%tags' => implode(', ', $this->getAllowedTags())]);
    }
    return $summary;
  }

  /**
   * Get allowed tags.
   *
   * @return array
   *   Returns allowed tags in array format.
   */
  private function getAllowedTags() {
    $allowed_tags = [];
    if (!empty($this->getSetting('allowed_tags'))) {
      $allowed_tags = explode("\r\n", str_replace([
        '<',
        '>'
      ], '', $this->getSetting('allowed_tags')));
    }
    return $allowed_tags;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $elements = [];
    $user = $this->entityTypeManager->getStorage('user');
    $cache = new BubbleableMetadata();
    $data['user'] = $user->load($this->currentUser->id());
    $data['node'] = $this->routeMatch->getParameter('node');

    foreach ($items as $delta => $item) {
      $value = $item->value;
      if (empty($value)) {
        $value = $item->getValue();
      }
      $value = $this->token->replace($value, $data, ['langcode' => 'en'], $cache);

      if (!empty($this->getAllowedTags())) {
        $elements[$delta]['#markup'] = $value;
        $elements[$delta]['#allowed_tags'] = $this->getAllowedTags();
      }
      else {
        $elements[$delta] = [
          '#children' => $value,
        ];
      }
    }

    return $elements;
  }

}
